# Generated by roxygen2: do not edit by hand

S3method("$",flexseq)
S3method("$",interval_index)
S3method("$",ordered_sequence)
S3method("$",priority_queue)
S3method("$<-",flexseq)
S3method("$<-",interval_index)
S3method("$<-",ordered_sequence)
S3method("$<-",priority_queue)
S3method("[",flexseq)
S3method("[",interval_index)
S3method("[",ordered_sequence)
S3method("[",priority_queue)
S3method("[<-",flexseq)
S3method("[<-",interval_index)
S3method("[<-",ordered_sequence)
S3method("[<-",priority_queue)
S3method("[[",flexseq)
S3method("[[",interval_index)
S3method("[[",ordered_sequence)
S3method("[[",priority_queue)
S3method("[[<-",flexseq)
S3method("[[<-",interval_index)
S3method("[[<-",ordered_sequence)
S3method("[[<-",priority_queue)
S3method(add_monoids,default)
S3method(add_monoids,flexseq)
S3method(add_monoids,interval_index)
S3method(add_monoids,ordered_sequence)
S3method(add_monoids,priority_queue)
S3method(as.list,flexseq)
S3method(as.list,interval_index)
S3method(as.list,ordered_sequence)
S3method(as.list,priority_queue)
S3method(as_flexseq,default)
S3method(as_flexseq,flexseq)
S3method(as_flexseq,interval_index)
S3method(as_flexseq,ordered_sequence)
S3method(as_flexseq,priority_queue)
S3method(c,flexseq)
S3method(c,ordered_sequence)
S3method(c,priority_queue)
S3method(coro::as_iterator,flexseq)
S3method(coro::as_iterator,priority_queue)
S3method(fapply,flexseq)
S3method(fapply,interval_index)
S3method(fapply,ordered_sequence)
S3method(fapply,priority_queue)
S3method(insert,default)
S3method(insert,interval_index)
S3method(insert,ordered_sequence)
S3method(insert,priority_queue)
S3method(length,flexseq)
S3method(length,interval_index)
S3method(length,ordered_sequence)
S3method(length,priority_queue)
S3method(locate_by_predicate,flexseq)
S3method(merge,flexseq)
S3method(merge,interval_index)
S3method(merge,ordered_sequence)
S3method(merge,priority_queue)
S3method(print,flexseq)
S3method(print,interval_index)
S3method(print,ordered_sequence)
S3method(print,priority_queue)
S3method(split_around_by_predicate,flexseq)
S3method(split_around_by_predicate,priority_queue)
S3method(split_at,flexseq)
S3method(split_at,priority_queue)
S3method(split_by_predicate,flexseq)
S3method(split_by_predicate,priority_queue)
S3method(str,flexseq)
S3method(unlist,flexseq)
export(add_monoids)
export(as_flexseq)
export(as_interval_index)
export(as_ordered_sequence)
export(as_priority_queue)
export(count_between)
export(count_key)
export(elements_between)
export(fapply)
export(flexseq)
export(get_measure)
export(get_measures)
export(insert)
export(insert_at)
export(interval_index)
export(locate_by_predicate)
export(loop)
export(lower_bound)
export(max_endpoint)
export(max_key)
export(max_priority)
export(measure_monoid)
export(min_endpoint)
export(min_key)
export(min_priority)
export(ordered_sequence)
export(peek_all_containing)
export(peek_all_key)
export(peek_all_max)
export(peek_all_min)
export(peek_all_overlaps)
export(peek_all_point)
export(peek_all_within)
export(peek_at)
export(peek_back)
export(peek_containing)
export(peek_front)
export(peek_key)
export(peek_max)
export(peek_min)
export(peek_overlaps)
export(peek_point)
export(peek_within)
export(plot_structure)
export(pop_all_containing)
export(pop_all_key)
export(pop_all_max)
export(pop_all_min)
export(pop_all_overlaps)
export(pop_all_point)
export(pop_all_within)
export(pop_at)
export(pop_back)
export(pop_containing)
export(pop_front)
export(pop_key)
export(pop_max)
export(pop_min)
export(pop_overlaps)
export(pop_point)
export(pop_within)
export(priority_queue)
export(push_back)
export(push_front)
export(split_around_by_predicate)
export(split_at)
export(split_by_predicate)
export(upper_bound)
export(validate_name_state)
export(validate_tree)
import(lambda.r)
importFrom(Rcpp,evalCpp)
importFrom(coro,loop)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,polygon)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
useDynLib(Immutables, .registration = TRUE)
