% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/40-priority_queue-queue-ops.R
\name{peek_all_min}
\alias{peek_all_min}
\title{Peek All Minimum-Priority Elements}
\usage{
peek_all_min(x)
}
\arguments{
\item{x}{A \code{priority_queue}.}
}
\value{
A \code{priority_queue} containing all minimum-priority elements in stable
queue order. Returns an empty queue when \code{x} is empty.
}
\description{
Returns the full minimum-priority tie run as a \code{priority_queue}.
}
\details{
The return is another \code{priority_queue()}, use \code{as.list()} to convert
the result to a standard R list.
}
\examples{
x <- priority_queue("a", "b", "c", priorities = c(2, 1, 1))
peek_all_min(x)
}
