% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/30-api-flexseq-tree-from.R
\name{tree_from}
\alias{tree_from}
\title{Build a Structural Tree from a Vector or List}
\usage{
tree_from(x, monoids = NULL)
}
\arguments{
\item{x}{Elements to insert.}

\item{monoids}{Optional named list of \code{measure_monoid} objects.}
}
\value{
A finger tree with cached measures for all monoids.
If \code{x} has names, they are used for name-based indexing and must be
complete (no missing/empty names) and unique.
}
\description{
Build a Structural Tree from a Vector or List
}
\keyword{internal}
