% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/30-api-flexseq-print.R
\name{print.flexseq}
\alias{print.flexseq}
\title{Print a flexseq}
\usage{
\method{print}{flexseq}(x, max_elements = 4L, show_custom_monoids = FALSE, ...)
}
\arguments{
\item{x}{A \code{flexseq}.}

\item{max_elements}{Maximum number of elements shown in preview (\code{head + tail}).}

\item{show_custom_monoids}{Logical; show attached non-default monoids and
their root cached measures.}

\item{...}{Passed through to per-element \code{print()}.}
}
\value{
The input \code{x}, returned invisibly. Called for its side effect
of printing a formatted preview of the sequence to the console.
}
\description{
Print a flexseq
}
\examples{
x <- as_flexseq(setNames(as.list(1:6), letters[1:6]))
print(x, max_elements = 4)

y <- as_flexseq(as.list(1:6))
print(y, max_elements = 3)
}
