% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/70-plotting.R
\name{get_graph_df}
\alias{get_graph_df}
\title{Build graph data frames for a finger tree}
\usage{
get_graph_df(t)
}
\arguments{
\item{t}{FingerTree.}
}
\value{
A list with three elements: \code{edge_df} (parent/child/label),
\code{node_df} (node/type/label), and \code{node_data} — a named list keyed by node
id whose values are lists with fields \code{type}, \code{label}, \code{measures} (cached
monoid values for structural nodes; \code{NULL} for element leaves), and
\code{element} (the raw leaf entry for element nodes; \code{NULL} for structural
nodes).
}
\description{
Build graph data frames for a finger tree
}
\keyword{internal}
