% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/30-api-flexseq-print.R
\name{print.FingerTree}
\alias{print.FingerTree}
\alias{print.Deep}
\alias{print.Single}
\alias{print.Empty}
\title{Print a compact summary of a finger tree}
\usage{
\method{print}{FingerTree}(x, max_elements = 4L, show_custom_monoids = FALSE, ...)

\method{print}{Deep}(x, ...)

\method{print}{Single}(x, ...)

\method{print}{Empty}(x, ...)
}
\arguments{
\item{x}{FingerTree.}

\item{max_elements}{Maximum number of elements shown in preview (\code{head + tail}).
Default \code{4}.}

\item{show_custom_monoids}{Logical; show attached non-default monoids and
their root cached measures. Default \code{FALSE}.}

\item{...}{Passed through to \code{print()} for preview elements.}
}
\value{
\code{x}, invisibly.
}
\description{
Print a compact summary of a finger tree
}
\examples{
x <- as_flexseq(setNames(as.list(1:6), letters[1:6]))
print(x, max_elements = 4)
sum_m <- measure_monoid(`+`, 0, function(el) as.numeric(el))
print(add_monoids(as_flexseq(1:3), list(sum = sum_m)), max_elements = 0, show_custom_monoids = TRUE)

y <- as_flexseq(as.list(1:6))
print(y, max_elements = 3)
}
\keyword{internal}
