% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/40-priority_queue-queue-ops.R
\name{max_priority}
\alias{max_priority}
\title{Maximum Priority Value}
\usage{
max_priority(x)
}
\arguments{
\item{x}{A \code{priority_queue}.}
}
\value{
Maximum priority value, or \code{NULL} when \code{x} is empty.
}
\description{
Returns the current maximum priority scalar in the queue.
}
\details{
Uses cached \code{.pq_max} monoid state.
}
\examples{
q <- priority_queue("a", "b", priorities = c(2, 1))
max_priority(q)
max_priority(priority_queue())
}
