% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/50-ordered_sequence-ops.R
\name{min_key}
\alias{min_key}
\title{Minimum Key Value}
\usage{
min_key(x)
}
\arguments{
\item{x}{An \code{ordered_sequence}.}
}
\value{
Minimum key, or \code{NULL} when \code{x} is empty.
}
\description{
Returns the smallest key currently present in the ordered sequence.
}
\details{
This follows sequence key order directly.
}
\examples{
x <- ordered_sequence("a", "b", keys = c(2, 1))
min_key(x)
min_key(ordered_sequence())
}
